import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

class Production implements Comparable<Production> {
	public final NonTerminal left;
	public final List<Symbol> right;

	Production(NonTerminal left, Symbol right) {
		this(left, Collections.singletonList(right));
	}

	Production(NonTerminal left, Symbol... right) {
		this(left, Arrays.asList(right));
	}

	Production(NonTerminal left, List<? extends Symbol> right) {
		this.left = left;
		if (right.isEmpty()) {
			this.right = Collections.emptyList();
		} else if (right.size() == 1) {
			this.right = Collections.singletonList(right.get(0));
		} else {
			this.right = Collections.unmodifiableList(new ArrayList<>(right));
		}
	}

	public static List<String> collectionToString(Collection<?> list) {
		return list.stream().map(Object::toString).collect(Collectors.toList());
	}

	@Override
	public String toString() {
		return left + " -> " + (right.isEmpty() ? "" : String.join(" ", collectionToString(right)));
	}

	@Override
	public boolean equals(Object o) {
		if (this == o) {
			return true;
		}
		if (!(o instanceof Production)) {
			return false;
		}
		Production that = (Production) o;
		return Objects.equals(left, that.left) && Objects.equals(right, that.right);
	}

	@Override
	public int hashCode() {
		return Objects.hash(left, right);
	}

	@Override
	public int compareTo(Production o) {
		int compare = left.compareTo(o.left);
		return compare != 0 ? compare : Util.compareLexicographic(right.iterator(), o.right.iterator());
	}

}
